const EventEmitter = require('eventemitter3');

const BattleCalc = require('../../BattleCalc');
const StandardStrike = require('../StandardStrike');
const DamageEvent = require('../../events/DamageEvent');
const BleedingCondition = require('../../conditions/BleedingCondition');
const { logError } = require('../../logger');

const Slash = function(battleState) {
  this.handle = 'slash';
  this.physical = true;
  this.weapon = true;
  this.melee = true;
  this.harm = true;

  this.standardStrike = new StandardStrike(this, battleState);

  this.eventEmitter = new EventEmitter();

  this.isLegal = function(actor, victim) {
    return victim.unit;
  };

  this.execute = function(actor, victim) {
    if (!actor || !actor.inPlay || !victim || (victim.unit&&!victim.inPlay)) {
      return { success: false };
    }

    try {
      var ability = BattleCalc.getEquippedAbility(actor, this.handle);

      var result = this.standardStrike.execute(
        actor,
        victim,
        ability.damage,
        true,
      );
      result.abilityHandle = this.handle;

      if (result.success && !victim.dead) {
        var bleedDamageDummy = new DamageEvent(battleState, actor, ability.potency, this);
        bleedDamageDummy.applyStanceAdjustments(victim);

        // if the unit is already bleeding, simply add to the potency of the extant condition
        var bleeding = BattleCalc.getCondition(victim, 'bleeding');
        if (bleeding) {
          bleeding.potency += bleedDamageDummy.damage;
          battleState.updateCondition(victim.uid, bleeding.uid);
        } else {
          battleState.onUnitCondition(new BleedingCondition(
            actor,
            victim,
            battleState,
            bleedDamageDummy.damage,
          ));
        }
      }


      result.abilityHandle = this.handle;
      result.actorId = actor.uid;
      result.victimId = victim? victim.uid : null;
      result.allPieces = battleState.allPieces;
      this.eventEmitter.emit('result', result);
    } catch (err) {
      logError(err, {
        module: 'Slash',
        func: 'execute',
        actor,
        victim,
      });

      return { success: false };
    }
  };
};
module.exports = Slash;
