/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const MoveDecision = require('./MoveDecision');
const BattleCalc = require('../../BattleCalc');

const MoveAwayDecision = function(actor, battleState) {
  var node = new MoveDecision(actor, battleState);

  node.maxJump = node.jumpRange;

  node.query = function(input) {
    if (!input) {
      return null;
    }

    if (input === this.actor) {
      return null;
    }

    if (input === this.battleState.getTileAtCoords(this.actor.x,this.actor.y)) {
      return null;
    }

    this.pieceToFleeFrom = input;
    this.target = this.getRelevantTile(input);

    this.targetDist = BattleCalc.boardDistance(this.actor, this.target);
    this.jumpRange = this.calculateMaxJumpRange();

    if (!this.target || this.targetDist===0) {
      const adjacentTiles = this.battleState.getAdjacentTiles(this.actor);
      var candidates = [];
      for (var i = 0; i < adjacentTiles.length; ++i)
        var tile = adjacentTiles[ i ];
      if (!tile.occupied && BattleCalc.boardDistance(tile,this.pieceToFleeFrom)>BattleCalc.boardDistance(this.actor,this.pieceToFleeFrom)) {
        candidates.push(tile);
      }

      if (candidates.length > 0)	{
        return this.branches[this.NO_JUMP_ABILITY].query(candidates[ Math.floor(Math.random()*candidates.length) ]);
      } else {
        var randomAdjacentTile;
        var attempts = 0;
        while (!randomAdjacentTile || randomAdjacentTile.occupied) {
          randomAdjacentTile = adjacentTiles[ Math.floor(Math.random()*candidates.length) ];
          if (++attempts > 10) {
            return null;
          }
        }

        return this.branches[this.NO_JUMP_ABILITY].query(randomAdjacentTile);
      }
    }
    else if (this.jumpRange===0
				||	this.targetDist < 3
				||	this.targetBeyondJumpRange())
    {
      return this.branches[this.NO_JUMP_ABILITY].query(this.getBestAdjacentTile(this.battleState));
    }
    else
    {
      return this.branches[this.WITHIN_JUMP_RANGE].query(this.target);
    }
  };

  node.getRelevantTile = function(target) {
    const candidateTiles = [];

    for (var prop in this.battleState.allTiles) {
      var tile = this.battleState.allTiles[ prop ];
      if (!tile.occupied && BattleCalc.pieceIsBetweenPair(this.actor,[this.pieceToFleeFrom,tile])) {
        candidateTiles.push(tile);
      }
    }

    if (candidateTiles.length === 0) {
      return null;
    } else {
      return candidateTiles[ Math.floor(Math.random()*candidateTiles.length) ];
    }
  };

  return node;
};
module.exports = MoveAwayDecision;
