const ItemGenerator = require('../../../game_items/ItemGenerator.js');
const Bow = require('../../../game_items/equipment/off_hand/Bow.js');
const MedTorsoe = require('../../../game_items/equipment/torsoe/MedTorsoe.js');
const getMobEquipmenAssignmentFromBags = require('./getMobEquipmenAssignmentFromBags');

const skeletal_archer = (level) => {
  const bags = [{
    items: [
      ItemGenerator.getRandomItem(MedTorsoe, 0, Math.min(1, 0.1 + level / 100)), // torsoe
      ItemGenerator.getRandomItem(Bow, 0, Math.min(1, 0.1 + level / 100)), // off_hand bow
    ],
  }];

  return {
    handle: 'skeletal_archer',
    level,
    getAbilityTrees: ((bound_level) => ([{
      handle: 'skeletal_archer_abilities',
      abilities: {
        move: { handle: 'move', level: 1 },
        melee_attack: { handle: 'melee_attack', level: 1 },
        arrow_shot: { handle: 'arrow_shot', level: Math.min(10, Math.round(1 + bound_level / 2)) },
      },
    }])).bind(null, level),
    getBaseAttributes: ((bound_level) => ({
      learning: Math.round(1.1 + bound_level * 0.3 + Math.random() * 3),
      perception: Math.round(1.1 + bound_level * 0.55 + Math.random() * 3),
      mana: Math.round(1.1 + bound_level * 0.3 + Math.random() * 3),
      toughness: Math.round(0.9 + bound_level * 0.2 + Math.random() * 3),
      strength: Math.round(1.1 + bound_level * 0.55 + Math.random() * 3),
      willpower: Math.round(1.1 + bound_level * 0.3 + Math.random() * 3),
      dexterity: Math.round(1.2 + bound_level * 0.85 + Math.random() * 3),
    })).bind(null, level),
    getDefaultAIConfig: () => (['dps_ranged', 'interference', 'dps_close', 'tank', 'support']),
    xpValue: level * 55,
    bags,
    build: {
      ai: {
        priorities: ['dps_ranged', 'interference', 'dps_close', 'tank', 'support'],
        settings: {
          chanceToSkipRoles: true,
          autoMove: true,
          waitForAPMP: false,
        },
      },
      ai_unit: true,
      beast: false,
      equipment_assignment: getMobEquipmenAssignmentFromBags(bags),
      equipped_abilities: ['move', 'melee_attack', 'arrow_shot'],
      handle: 'skeletal_archer',
      humanoid: true,
      level,
      team: 'black',
      undead: true,
      unit: true,
    },
  };
};
module.exports = skeletal_archer;
