'use strict';

const assert = require('assert');
const { getMongoDBClient } = require('../index');

const getTeamForMatchType = async ({
  playerId,
  game_mode,
  game_submode,
  attack_loadout,
}) => {
  const { db } = await getMongoDBClient();

  const { loadouts } = await db.collection('script.loadouts')
    .findOne(
      { _id: playerId },
      {
        projection: { [`loadouts.${game_mode}.${game_submode}`]: 1 },
      },
    );
  assert.ok(loadouts);

  const competition_profiles_res = await db.collection('script.competition_profiles').findOne(
    { _id: playerId },
    {
      projection: {
        [`competition_profiles.${game_submode}`]: 1,
      },
    });

  const loadout = loadouts[game_mode][game_submode];
  const team_handles = (attack_loadout || loadout)
    .filter(({ engagement }) => !!engagement)
    .sort((a, b) => a.engagement.slot - b.engagement.slot)
    .map(({ handle }) => handle);

  // get a subset of the player's full roster, including only those on this matchType's team
  const roster_projection = {};
  for (const handle of team_handles) {
    roster_projection[`hero_roster.${handle}`] = 1;
  }
  const { hero_roster } = await db.collection('script.heroRosters')
    .findOne(
      { _id: playerId },
      { projection: roster_projection },
    );

  // get a subset of the player's full roster, including only those on this matchType's team
  const hero_builds_projection = {};
  for (const handle of team_handles) {
    hero_builds_projection[`hero_builds.${handle}`] = 1;
  }
  const { hero_builds } = await db.collection('script.hero_builds')
    .findOne(
      { _id: playerId },
      { projection: hero_builds_projection },
    );

  const player = await db.collection('player')
    .findOne(
      { _id: playerId },
      {
        projection: {
          'scriptData.displayName': 1,
          'scriptData.chat': 1,
        },
      },
    );

  const result = {
    builds: Object.keys(hero_builds).reduce((result, hero_handle) => {
      result[hero_handle] = hero_builds[hero_handle].find(({ _id }) => {
        const { hero_build_id } = (attack_loadout || loadout).find(({ handle }) => handle === hero_handle);
        return hero_build_id === _id;
      });
      return result;
    }, {}),
    competition_profile: competition_profiles_res?.competition_profiles
      ? competition_profiles_res?.competition_profiles[game_submode]
      : null,
    loadout,
    player,
    positions: team_handles,
    roster: hero_roster,
  };

  return result;
};
module.exports = getTeamForMatchType;
